#include "gui.h"
#include "guiGlobal.h"


void gui::startup() {

	Woopsi::startup();

	// Setting up the screens
	fileScreen = new AmigaScreen("File Listing", Gadget::GADGET_DRAGGABLE, AmigaScreen::AMIGA_SCREEN_SHOW_DEPTH | AmigaScreen::AMIGA_SCREEN_SHOW_FLIP);
	woopsiApplication->addGadget(fileScreen);

	connectScreen = new AmigaScreen("Connect Screen", Gadget::GADGET_DRAGGABLE, AmigaScreen::AMIGA_SCREEN_SHOW_DEPTH | AmigaScreen::AMIGA_SCREEN_SHOW_FLIP);
	woopsiApplication->addGadget(connectScreen);

	keyScreen = new AmigaScreen("Keyboard", Gadget::GADGET_DRAGGABLE, AmigaScreen::AMIGA_SCREEN_SHOW_DEPTH | AmigaScreen::AMIGA_SCREEN_SHOW_FLIP);
	woopsiApplication->addGadget(keyScreen);

	keyboard = new WoopsiKeyboard(0, 20, 256, 172, "Keyboard", Gadget::GADGET_DRAGGABLE, AmigaWindow::AMIGA_WINDOW_SHOW_DEPTH);
	keyScreen->addGadget(keyboard);

	transferScreen = new AmigaScreen("Transfering...", Gadget::GADGET_DRAGGABLE, AmigaScreen::AMIGA_SCREEN_SHOW_DEPTH | AmigaScreen::AMIGA_SCREEN_SHOW_FLIP);
	woopsiApplication->addGadget(transferScreen);
	
	

	SuperBitmap* logo = new SuperBitmap(0, 13, 256, 192, 256, 192, true);
	logo->drawBitmap(0, 0, 256, 192, background_Bitmap, 0, 0, 256, 192);
	connectScreen->addGadget(logo);

	connectScreen->flipToTopScreen();

	// Create listboxes
	remoteFiles = new ScrollingListBox(131, 30, 125, 160);
	localFiles = new ScrollingListBox(0, 30, 120, 160);

	fileScreen->addGadget(remoteFiles);
	fileScreen->addGadget(localFiles);

	//Creating Gadgets
	download = new Button(121, 80, 10, 10, "<");
	upload = new Button(121, 110, 10, 10, ">");
	connect = new Button(0, 15, 80, 20, "Connect");
	pwLbl = new Label(0, 150, 80, 20, "Password:");
	loginLbl = new Label(0, 110, 80, 20, "Login:");
	serverConnect = new Label(0, 50, 200, 20, "Server address:");
	disconnect = new Button(100, 60, 80, 20, "Disconnect");
	local = new Label(0, 15, 105, 12, "Local Files:");
	remote = new Label(131, 15, 108, 12, "Remote Files:");
	serverText = new TextBox(0, 70, 200, 20, "");
	serverText->setBackColour(woopsiRGB(32, 178, 170));
	loginText = new TextBox(0, 130, 200, 20, "");
	passwordText = new TextBox(0, 170, 200, 20, "");
	curRemoteDirLbl = new Label(0, 150, 256, 20, "Current Remote:");
	curLocalDirLbl = new Label(0, 15, 256, 20, "Current Local:");
	curRemoteText = new TextBox(0, 170, 256, 20, "");
	curLocalText = new TextBox(0, 35, 256, 20, "");
	localDirUp = new Button(106, 15, 10, 10, "^");
	remoteCdup = new Button(240, 15, 10, 10, "^");
	fileLbl = new Label(0, 15, 105, 12, "");
	ofLbl = new Label(0, 30, 105, 12, "of");
	transferLbl = new Label(0, 45, 105, 12, "");

	//Check to see if we have existing settings saved to file from a previous
	//ftp connection
	FILE* configfile = fopen ("fileNinja.inf", "r+");
	if(configfile){
		char writeserver[64];
		char writelogin[64];
		char writepass[64];
		if(fscanf(configfile,"%63s\n%63s\n%63s\n",writeserver, writelogin,writepass) == 3){
			serverText->setText(writeserver);
			loginText->setText(writelogin);
			passwordText->setText(writepass);
			fclose(configfile);
		}
	}


	//Adding gadgets to the window
	fileScreen->addGadget(download);
	fileScreen->addGadget(upload);
	fileScreen->addGadget(local);
	fileScreen->addGadget(remote);
	fileScreen->addGadget(remoteCdup);
	fileScreen->addGadget(localDirUp);
	transferScreen->addGadget(transferLbl);
	transferScreen->addGadget(ofLbl);
	transferScreen->addGadget(fileLbl);

	//Hiding controls until a connection is made
	disconnect->shelve();
	curRemoteDirLbl->shelve();
	curRemoteText->shelve();
	curLocalDirLbl->shelve();
	curLocalText->shelve();
	fileScreen->hide();
	transferScreen->hide();

	//Used for keyboard writing, initially set to 1 for serverText TextBox
	connectFocus = 1;

	//Setting event handlers
	connect->setEventHandler(this);
	keyboard->setEventHandler(this);
	disconnect->setEventHandler(this);
	download->setEventHandler(this);
	upload->setEventHandler(this);
	remoteCdup->setEventHandler(this);
	localDirUp->setEventHandler(this);
	remoteFiles->setEventHandler(this);
	localFiles->setEventHandler(this);

	enableDrawing();	// Ensure Woopsi can now draw itself
	draw();				// Draw initial state

	popup = new Alert(0, 13, 256, 80, "Connecting", "Initializing wireless connection from firmware...");
	connectScreen->addGadget(popup);
	if( wifi_connect() == 1)
	{
		popup->close();
		popup = new Alert(50, 50, 200, 80, "Connected", "Successfully associated with AP!");
		keyScreen->addGadget(popup);
	}
	else 
	{
		popup->close();
		popup = new Alert(50, 50, 200, 80, "Error", "Problem encountered associating with access point.");
		keyScreen->addGadget(popup);
	}
	connectScreen->addGadget(connect);
	connectScreen->addGadget(serverConnect);
	connectScreen->addGadget(loginText);
	connectScreen->addGadget(loginLbl);
	connectScreen->addGadget(pwLbl);
	connectScreen->addGadget(passwordText);
	connectScreen->addGadget(serverText);
	connectScreen->addGadget(disconnect);
	connectScreen->addGadget(curLocalDirLbl);
	connectScreen->addGadget(curRemoteDirLbl);
	connectScreen->addGadget(curRemoteText);
	connectScreen->addGadget(curLocalText);
	//test of progress bar
	/*progress = new ProgressBar(0, 20, 255, 20);
	connectScreen->addGadget(progress);
	progress->setMaximumValue((s16)100);
	progress->setMinimumValue(0);
	progress->setValue(0);*/
	
	/*Upload functionality has a bug that is being worked out, disabling
	upload->disable();*/

	connectScreen->draw();
	
	//This variable is only needed when progress bar functionality is working
	//still under development
	guiInstance = this;
}


void gui::shutdown() {

	Woopsi::shutdown();
}

bool gui::handleEvent(const EventArgs& e) {

	// Check that a valid gadget fired the event
	if (e.gadget != NULL) {
		//Upload Button
		if (e.gadget == upload)// Check which gadget fired the event
		{
			if (e.type == EVENT_ACTION)// Check which event we've got to process
			{						   
				int index = 0;
				while((index = localFiles->getSelectedIndex()) >= 0){
					if( localDir[index]->isdir )
					{
						popup = new Alert(50, 50, 200, 80, "Directory Selected", "Cannot upload directory.");
						fileScreen->addGadget( popup );
					}
					else
					{
						popup = new Alert(50, 50, 200, 80, "Uploading file:", localFiles->getOption(index)->text);
						connectScreen->addGadget( popup );
						string localPath(curLocalText->getText());
						localPath += localFiles->getOption(index)->text + NULL;
						if(!FtpPut(localPath.c_str(), localFiles->getOption(index)->text,'I',conn)){
							popup->close();
							popup = new Alert(50, 50, 200, 80, "Failed to put file", localFiles->getOption(index)->text);
							connectScreen->addGadget( popup );
						}
						else
						{
							popup->close();
						}
						remoteFiles->removeAllOptions();
						//Freeing memory
						free_fileinfo(remoteDir);
						remoteDir = get_remote_dir(NULL, conn);
						getFileListing(remoteDir, remoteFiles);
					}
					localFiles->deselectOption(index);
				}						
			}
			return true;
		}
		//Download Button
		else if (e.gadget == download)
		{
			if (e.type == EVENT_ACTION)
			{
				int index = 0;
				while((index = remoteFiles->getSelectedIndex()) >= 0)
				{
					if( remoteDir[index]->isdir )
					{
						popup = new Alert(50, 50, 200, 80, "Directory Selected", "Cannot download directory.");
						fileScreen->addGadget( popup );
					}
					else
					{
						popup = new Alert(50, 50, 200, 80, "Downloading file:", remoteFiles->getOption(index)->text);
						connectScreen->addGadget( popup );
						//Progress bar doesnt work right yet....
						//progress = new ProgressBar(5, 20, 200, 20);
						//fileScreen->addGadget(progress);
						//progress->setMaximumValue(remoteDir[index]->filesize %100);
						//progress->setMinimumValue(0);
						//progress->setFillColour(woopsiRGB(102, 255, 255));
						//progress->setShineColour(woopsiRGB(102, 255, 255));
						//progress->setValue(0);
						
						fileScreen->hide();
						string localPath(curLocalText->getText());
						localPath += remoteFiles->getOption(index)->text + NULL;
						char size[1024];
						sprintf(size, "%d", remoteDir[index]->filesize);
						transferLbl->setText(size);
						transferScreen->flipToBottomScreen();
						transferScreen->show();
						draw();
						if(!FtpGet(localPath.c_str(),remoteFiles->getOption(index)->text,'I',conn)){
							popup->close();
							popup = new Alert(50, 50, 200, 80, "Failed to get file", remoteFiles->getOption(index)->text);
							connectScreen->addGadget( popup );
						}
						else
						{
							popup->close();
						}
						popup = new Alert(50, 50, 200, 80, "Download Complete", remoteFiles->getOption(index)->text);
						fileScreen->addGadget(popup);
						//progress->close();
						localFiles->removeAllOptions();
						//Freeing memory
						free_fileinfo(localDir);
						localDir = get_dir((char*)curLocalText->getText());
						getFileListing(localDir, localFiles);
					}

					remoteFiles->deselectOption(index);
					transferLbl->setText("");
					fileLbl->setText("");
				}
			}	
			fileScreen->show();
			transferScreen->hide();
			draw();
			return true;	
		}
		//Remote file directory change directory up one level
		else if(e.gadget == remoteCdup)
		{
			if(e.type == EVENT_ACTION)
			{
				if( FtpCDUp(conn) )
				{
					popup = new Alert(50, 50, 200, 80, "Change Directory", "Attempting to change directory...");
					fileScreen->addGadget(popup);
					remoteFiles->removeAllOptions();
					//Freeing memory
					free_fileinfo(remoteDir);
					remoteDir = get_remote_dir(NULL, conn);
					getFileListing(remoteDir, remoteFiles);
					if( FtpPwd(curRemote, 256, conn) )
					{
						curRemoteText->setText(curRemote);
					}
					popup->close();

					fileScreen->draw();
				}
				else
				{
					popup = new Alert(50, 50, 200, 80, "Error", "Remote cdup failed.");
					fileScreen->addGadget(popup);
				}
			}
			return true;
		}
		//Local cdup
		else if(e.gadget == localDirUp)
		{
			if(e.type == EVENT_ACTION)
			{

				if(strcmp("/\0",curLocalText->getText()) == 0)
				{
					popup = new Alert(50, 50, 200, 80, "Change Directory", "Currently at root directory");
					fileScreen->addGadget(popup);
					return true;
				}
				else
				{
					localCdUp();
				}
			}
			return true;
		}
		else if (e.gadget == connect)
		{
			if (e.type == EVENT_ACTION)
			{
				string errorMsg = "Failed: ";
				bool connected = false;
				//When you hit connect, it writes the connection info to fileNinja.inf
				FILE* configfile = fopen ("fileNinja.inf", "w+");
				if(configfile){
					fprintf(configfile,"%s\n%s\n%s\n",serverText->getText(),loginText->getText(),passwordText->getText());
					fclose(configfile);
				}

				if( !FtpConnect(serverText->getText(),&conn))
				{
					connected = false;
					errorMsg += "ftpcon, ";
				}
				else connected = true;

				if(connected && !FtpLogin(loginText->getText(),passwordText->getText(), conn))
				{
					connected = false;
					errorMsg += "ftpLogin.";
				}
				if( connected )
				{
					connect->shelve();
					pwLbl->shelve();
					passwordText->shelve();
					loginLbl->shelve();
					loginText->shelve();
					serverConnect->shelve();
					serverText->shelve();
					disconnect->unshelve();
					keyScreen->hide();
					fileScreen->show();
					draw();
					connectScreen->flipToTopScreen();
					fileScreen->flipToBottomScreen();
					curRemoteDirLbl->unshelve();
					curRemoteText->unshelve();
					curLocalDirLbl->unshelve();
					curLocalText->unshelve();

					//Getting the local file listing
					localDir = get_dir("/");
					getFileListing(localDir, localFiles);

					//Getting the remote file listing
					remoteDir = get_remote_dir(NULL, conn);
					getFileListing(remoteDir, remoteFiles);
					if( FtpPwd(curRemote, 256, conn) )
					{
						curRemoteText->setText(curRemote);
					}
					curLocalText->setText("/");
					draw();
				}
				else
				{
					popup = new Alert(50, 50, 200, 80, "Connect Failed", errorMsg.c_str());
					connectScreen->addGadget( popup );
				}	
			}
			return true;	
		}
		else if (e.gadget == disconnect)
		{
			if (e.type == EVENT_ACTION)
			{

				FtpQuit(conn);


				connect->unshelve();
				pwLbl->unshelve();
				passwordText->unshelve();
				loginLbl->unshelve();
				loginText->unshelve();
				serverConnect->unshelve();
				serverText->unshelve();
				disconnect->shelve();
				keyScreen->show();
				fileScreen->hide();
				keyScreen->flipToBottomScreen();
				connectScreen->flipToTopScreen();
				localFiles->removeAllOptions();
				remoteFiles->removeAllOptions();
				curRemoteDirLbl->shelve();
				curRemoteText->setText("");
				curRemoteText->shelve();
				curLocalDirLbl->shelve();
				curLocalText->shelve();

				//Freeing up dynamically allocated memory
				free_fileinfo(localDir);
				free_fileinfo(remoteDir);

				draw();	
			}
			return true;
		}
		//Double tap in remote file list box, changes directory if item is a directory
		else if(e.gadget == remoteFiles)
		{
			if(e.type == EVENT_DOUBLE_CLICK)
			{
				int index = (int)remoteFiles->getSelectedIndex();

				if(index >= 0)
				{
					if( remoteDir[index] && remoteDir[index]->isdir )
					{
						if(FtpChdir( remoteDir[index]->filename, conn))
						{
							popup = new Alert(50, 50, 200, 80, "Change Directory", "Attempting to change directory...");
							fileScreen->addGadget(popup);
							remoteFiles->removeAllOptions();
							fileScreen->draw();
							//Freeing memory
							free_fileinfo(remoteDir);
							remoteDir = get_remote_dir(NULL, conn);
							if( FtpPwd(curRemote, 1024, conn) )
							{
								curRemoteText->setText(curRemote);
							}
							getFileListing(remoteDir, remoteFiles);
							popup->close();

							fileScreen->draw();
						}
						else
						{
							popup = new Alert(50, 50, 200, 80, "Error", "Change directory failed.");
							fileScreen->addGadget(popup);
						}
					}
					else
					{
						popup = new Alert(50, 50, 200, 80, "Error", "Not a directory.");
						fileScreen->addGadget(popup);
					}
				}
				else
				{
					popup = new Alert(50, 50, 200, 80, "Error", "Nothing selected.");
					fileScreen->addGadget(popup);
				}
			}
			return true;
		}
		//Double tap in local files list box, changes dir if item is a dir
		else if(e.gadget == localFiles)
		{
			if(e.type == EVENT_DOUBLE_CLICK)
			{
				int index = (int)localFiles->getSelectedIndex();

				if(index >= 0)
				{
					if( localDir[index] && localDir[index]->isdir )
					{
						////test
						//int blah = progress->getValue();
						//char test[10];
						//sprintf(test, "Value: %i\0", blah);
						//popup = new Alert(50, 50, 200, 80, "Test", test);
						//fileScreen->addGadget(popup);
						//progress->setValue(progress->getValue() + (s16)10);
						////
						string dir;
						string newDir(localDir[index]->filename);
						if( newDir.compare(".\0") != 0)
						{
							if( newDir.compare("..\0") == 0)
							{
								localCdUp();
								return true;
							}
							else if(newDir.compare("/\0") == 0)
							{
								popup = new Alert(50, 50, 200, 80, "Change Directory", "Currently at root directory");
								fileScreen->addGadget(popup);
								return true;
							}
							else
							{
								dir = curLocalText->getText() + NULL;
								dir += newDir + "/";
							}
							//Freeing memory
							free_fileinfo(localDir);
							if((localDir = get_dir((char*)dir.c_str())) != 0)
							{
								localFiles->removeAllOptions();
								fileScreen->draw();
								getFileListing(localDir, localFiles);
								curLocalText->setText(dir.c_str());
								fileScreen->draw();
							}
							else
							{
								popup = new Alert(50, 50, 200, 80, "Error", "Change directory failed.");
								fileScreen->addGadget(popup);
							}
						}
					}
					else
					{
						popup = new Alert(50, 50, 200, 80, "Error", "Not a directory.");
						fileScreen->addGadget(popup);
					}
				}
				else
				{
					popup = new Alert(50, 50, 200, 80, "Error", "Nothing selected.");
					fileScreen->addGadget(popup);
				}
			}
			return true;
		}
		else if (e.gadget == keyboard)
		{
			if(e.type == EVENT_ACTION)
			{

				key = keyboard->getLastKeyClicked();
				int curKey = key->getKeyType();

				//Enter key clicked
				if( curKey == 3 )
				{
					connectFocusChange();
				}
				//Backspace was clicked
				else if( curKey == 7 )
				{
					switch( connectFocus )
					{
					case 1: serverText->setText((deleteBack((char *)serverText->getText())));
						break;
					case 2: loginText->setText(deleteBack((char *)loginText->getText()));
						break;
					case 3: passwordText->setText(deleteBack((char *)passwordText->getText()));
						break;
					}
				}
				//Space bar clicked
				else if( key->getKeyType() == 8 )
				{
					switch( connectFocus )
					{
					case 1: serverText->appendText(" ");
						break;
					case 2: loginText->appendText(" ");
						break;
					case 3: passwordText->appendText(" ");
						break;
					}
				}
				else
				{
					//Alphanumeric or symbol key clicked
					if( key->getKeyType() == 1)
					{
						switch( connectFocus )
						{
						case 1: serverText->appendText(key->getText());
							break;
						case 2: loginText->appendText(key->getText());
							break;
						case 3: passwordText->appendText(key->getText());
							break;
						}
					}
				}
			}
			return true;
		}
	}

	return false;
}

void gui::connectFocusChange()
{
	switch(connectFocus)
	{
	case 1: serverText->setBackColour(DefaultStyle::backColour);
		break;
	case 2: loginText->setBackColour(DefaultStyle::backColour);
		break;
	case 3: passwordText->setBackColour(DefaultStyle::backColour);
		break;
	}
	if( connectFocus == 3 )
	{
		connectFocus = 1;
	}
	else
	{
		connectFocus++;
	}

	switch(connectFocus)
	{
	case 1: serverText->setBackColour(woopsiRGB(32, 178, 170));
		break;
	case 2: loginText->setBackColour(woopsiRGB(32, 178, 170));
		break;
	case 3: passwordText->setBackColour(woopsiRGB(32, 178, 170));
		break;
	}
	connectScreen->draw();
}

char* gui::deleteBack( char *text )
{
	int len = strlen(text);
	if(len >= 0)
	{
		text[len - 1] = NULL;
	}
	return text;
}
void gui::getFileListing(fileinfo** output, ScrollingListBox* box)
{
	int i = 0;
	while(output[i])
	{
		if( output[i]->isdir )
		{
			box->addOption(output[i]->filename, i, box->getShineColour(), box->getBackColour(), box->getShineColour(), box->getHighlightColour());
		}
		else
		{
			box->addOption(output[i]->filename, i);
		}
		i++;
	}
}

void gui::localCdUp()
{
	curLocalText->setText(deleteBack((char*)curLocalText->getText()));
	string dir(curLocalText->getText());
	size_t index = dir.find_last_of("/");
	dir = dir.substr(0, index+1);
	//Freeing memory
	free_fileinfo(localDir);
	if((localDir = get_dir((char*)dir.c_str())) != 0)
	{
		localFiles->removeAllOptions();
		fileScreen->draw();
		getFileListing(localDir, localFiles);
		curLocalText->setText(dir.c_str());
		fileScreen->draw();
	}
	else
	{
		popup = new Alert(50, 50, 200, 80, "Error", "Change directory failed.");
		fileScreen->addGadget(popup);
	}
}
void gui::setProgress(int value)
{
	char bytes[1024];
	sprintf(bytes, "%d", value);
	fileLbl->setText(bytes);
	//progress->setValue(progress->getValue() + value);
}

void callSetProgressWrapper(int value)
{
	guiInstance->setProgress(value);
}
